using System;
using System.Collections.Generic;
using CodePlex.DnD4eCampaignManager.DnD4e;
using CodePlex.DnD4eCampaignManager.DnD4e.Interfaces;
using NUnit.Framework;

namespace CodePlex.DnD4eCampaignManager.CampaignManagerTests.DnD4eTests
{
    [TestFixture]
    public class StoryArcTests
    {
        [Test]
        public void HasArcName()
        {
            const string arcName = "The Beginning";
            StoryArc storyArc = new StoryArc { Name = arcName };
            Assert.AreEqual(arcName, storyArc.Name);
        }

        [Test]
        public void HasComponentArcNames()
        {
            const string arcName = "The Beginning";
            List<string> componentArcNames = new List<string> { arcName };
            StoryArc storyArc = new StoryArc { ComponentArcNames = componentArcNames };
            Assert.AreEqual(componentArcNames, storyArc.ComponentArcNames);
        }

        [Test]
        public void ImplementsIEquatable()
        {
            Assert.IsInstanceOfType(typeof ( IEquatable<StoryArc> ), new StoryArc());
        }

        [Test]
        public void ImplementsISenses()
        {
            Assert.IsInstanceOfType(typeof ( IStoryArc ), new StoryArc());
        }

        [Test]
        public void IsEquatable()
        {
            StoryArc primaryDomainObject = new StoryArc { Name = "Starting Out" };
            StoryArc duplicateDomainObject = new StoryArc { Name = "Starting Out" };
            StoryArc differentDomainObject = new StoryArc { Name = "Deep in the Dungeon" };

            TestHelpers.TestEquatabilityExcludingOperators(primaryDomainObject, duplicateDomainObject, differentDomainObject);
            Assert.IsTrue(primaryDomainObject == duplicateDomainObject);
            Assert.IsTrue(primaryDomainObject != differentDomainObject);
        }
    }
}